﻿Imports Infragistics.Web.UI

Public Class CustomBooleanConverter
    Implements GridControls.IBooleanConverter, IStateManager


#Region "--- Code for implementing IBooleanConverter ---"
    Private msColumnName As String = String.Empty
    Private moDefaultFalseValue As Object
    Private moDefaultTrueValue As Object
    Private mbNullIsFalse As Boolean = True


    Public Sub New()
        _viewstate = New StateBag()
    End Sub


    ''' <summary>
    ''' Default Constructor (Assumes that NULL/NOTHING == FALSE)
    ''' </summary>
    ''' <param name="poDefaultFalseValue"></param>
    ''' <param name="poDefaultTrueValue"></param>
    ''' <remarks></remarks>
    Public Sub New(poDefaultFalseValue As Object, poDefaultTrueValue As Object, psColumnName As String)
        _viewstate = New StateBag()
        SetDefaultFalseValue(poDefaultFalseValue)
        SetDefaultTrueValue(poDefaultTrueValue)
        NullIsFalse = True
        ColumnName = psColumnName
    End Sub


    ''' <summary>
    ''' Used when you want to have NULL/NOTHING != FALSE
    ''' </summary>
    ''' <param name="poDefaultFalseValue"></param>
    ''' <param name="poDefaultTrueValue"></param>
    ''' <param name="pbNullIsFalse"></param>
    ''' <remarks></remarks>
    Public Sub New(poDefaultFalseValue As Object, poDefaultTrueValue As Object, psColumnName As String, pbNullIsFalse As Boolean)
        _viewstate = New StateBag()
        SetDefaultFalseValue(poDefaultFalseValue)
        SetDefaultTrueValue(poDefaultTrueValue)
        NullIsFalse = pbNullIsFalse
        ColumnName = psColumnName
    End Sub


    Public Property NullIsFalse() As Boolean
        Get
            Return CStr(_viewstate("NullIsFalse"))
        End Get
        Set(ByVal value As Boolean)
            mbNullIsFalse = value
            _viewstate.SetItemDirty("NullIsFalse", True)
        End Set
    End Property


    Public Property ColumnName() As String
        Get
            Return CStr(_viewstate("ColumnName"))
        End Get
        Set(ByVal value As String)
            msColumnName = value
            _viewstate.SetItemDirty("ColumnName", True)
        End Set
    End Property


    Public ReadOnly Property DefaultFalseValue As Object Implements Infragistics.Web.UI.GridControls.IBooleanConverter.DefaultFalseValue
        Get
            Return _viewstate("DefaultFalseValue")
        End Get
    End Property


    Public Sub SetDefaultFalseValue(value As Object)
        moDefaultFalseValue = value
        _viewstate.SetItemDirty("DefaultFalseValue", True)
    End Sub


    Public ReadOnly Property DefaultTrueValue As Object Implements Infragistics.Web.UI.GridControls.IBooleanConverter.DefaultTrueValue
        Get
            Return _viewstate("DefaultTrueValue")
            'Return moDefaultTrueValue
        End Get
    End Property


    Public Sub SetDefaultTrueValue(value As Object)
        moDefaultTrueValue = value
        _viewstate.SetItemDirty("DefaultTrueValue", True)
    End Sub


    ''' <summary>
    ''' Defaults to True (IsFalse), Will be False (IsNOTFalse) when Type And Value do not match default OR when value is nothing and NullIsFalse boolean is also False
    ''' </summary>
    ''' <param name="value"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function IsFalse(value As Object) As Boolean Implements Infragistics.Web.UI.GridControls.IBooleanConverter.IsFalse
        Dim pbIsFalse As Boolean = True

        Try
            If value Is Nothing Then
                If Not mbNullIsFalse Then
                    pbIsFalse = False
                End If
            Else
                If value.GetType().Equals(DefaultFalseValue.GetType()) AndAlso Not value.Equals(DefaultFalseValue) Then
                    pbIsFalse = False
                End If
            End If

        Catch ex As Exception
            If Not mbNullIsFalse Then
                pbIsFalse = False
            End If
            Throw
        End Try

        Return pbIsFalse
    End Function


    ''' <summary>
    ''' Assumes False (IsNOTTrue), If value is not nothing, types are the same and values are a match, then True (IsTrue)
    ''' </summary>
    ''' <param name="value"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function IsTrue(value As Object) As Boolean Implements Infragistics.Web.UI.GridControls.IBooleanConverter.IsTrue
        Dim pbIsTrue As Boolean = False

        Try
            If value IsNot Nothing AndAlso value.GetType().Equals(DefaultTrueValue.GetType()) AndAlso value.Equals(DefaultTrueValue) Then
                pbIsTrue = True
            End If

        Catch ex As Exception
            Throw
        End Try

        Return pbIsTrue
    End Function
#End Region


#Region "--- Code for implementing IStateManager ---"
    Private _viewstate As StateBag


    Public ReadOnly Property IsTrackingViewState As Boolean Implements System.Web.UI.IStateManager.IsTrackingViewState
        Get
            Return CType(_viewstate, IStateManager).IsTrackingViewState
        End Get
    End Property


    Public Sub LoadViewState(savedState As Object) Implements System.Web.UI.IStateManager.LoadViewState
        If Not (savedState Is Nothing) Then
            CType(_viewstate, IStateManager).LoadViewState(savedState)
        End If
    End Sub


    Function SaveViewState() As Object Implements IStateManager.SaveViewState
        ' Check whether the message property exists in the ViewState property, and if it does, check
        ' whether it has changed since the most recent TrackViewState method call.
        If Not CType(_viewstate, IDictionary).Contains("DefaultTrueValue") OrElse _viewstate.IsItemDirty("DefaultTrueValue") OrElse _
            Not CType(_viewstate, IDictionary).Contains("DefaultFalseValue") OrElse _viewstate.IsItemDirty("DefaultFalseValue") OrElse _
            Not CType(_viewstate, IDictionary).Contains("ColumnName") OrElse _viewstate.IsItemDirty("ColumnName") OrElse _
            Not CType(_viewstate, IDictionary).Contains("NullIsFalse") OrElse _viewstate.IsItemDirty("NullIsFalse") Then
            '
            _viewstate.Clear()

            ' Add the _message property to the StateBag.
            _viewstate.Add("DefaultFalseValue", moDefaultFalseValue)
            _viewstate.Add("DefaultTrueValue", moDefaultTrueValue)
            _viewstate.Add("ColumnName", msColumnName)
            _viewstate.Add("NullIsFalse", mbNullIsFalse)
        End If

        Return CType(_viewstate, IStateManager).SaveViewState()
    End Function 'IStateManager.SaveViewState


    ''' <summary>
    ''' Implement the TrackViewState method for this class by calling the TrackViewState method of the class's private _viewstate property.
    ''' </summary>
    ''' <remarks></remarks>
    Sub TrackViewState() Implements IStateManager.TrackViewState
        CType(_viewstate, IStateManager).TrackViewState()
    End Sub 'IStateManager.TrackViewState


    ''' <summary>
    ''' Create a function that iterates through the view-state values stored for this class and returns the results as a string.
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function EnumerateViewState() As String
        Dim keyName, keyValue As String
        Dim result As String = [String].Empty
        Dim myStateItem As StateItem
        Dim myDictionaryEnumerator As IDictionaryEnumerator = _viewstate.GetEnumerator()

        While myDictionaryEnumerator.MoveNext()
            keyName = CStr(myDictionaryEnumerator.Key)
            myStateItem = CType(myDictionaryEnumerator.Value, StateItem)
            keyValue = CStr(myStateItem.Value)
            result = result + "<br>ViewState[" + keyName + "] = " + keyValue
        End While

        Return result
    End Function
#End Region
End Class
